import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'common/const.dart';

class G1SDK {
  static const envDev = 1;
  static const envStag = 2;
  static const envProd = 3;
  static const MethodChannel _channel = MethodChannel(Const.sdk);

  static void _log(String msg) {
    if (kDebugMode) {
      print(msg);
    }
  }

  static Future<String?> get platformVersion async {
    final String? version =
        await _channel.invokeMethod(Const.getPlatformVersion);
    return version;
  }

  static Future<bool> setupSDK(int env, String? writeKey,
      {bool isShowLog = false, bool isEnableIAM = true}) async {
    if (env != envDev && env != envStag && env != envProd) {
      return false;
    }
    return await _channel.invokeMethod(
      Const.setupTracking,
      {
        "env": env,
        "writeKey": writeKey,
        "isShowLog": isShowLog,
        "isEnableIAM": isEnableIAM,
      },
    );
  }

  static void trackEvent(String? workSpaceId, Map? identityId,
      List<Map>? profile, String? eventName, int? eventDate, Map? eventData,
      {Function(Object?)? onResponse, Function(Object?)? onFailure}) async {
    // print("workSpaceId $workSpaceId");
    // print("identityId $identityId");
    // print("profile profile");
    // print("eventName $eventName");
    // print("eventDate $eventDate");
    // print("eventData $eventData");
    final result = await _channel.invokeMethod<Map<dynamic, dynamic>>(
      Const.trackEvent,
      {
        "workSpaceId": workSpaceId,
        "identityId": identityId,
        "profile": profile,
        "eventName": eventName,
        "eventDate": eventDate,
        "eventData": eventData,
      },
    );
    if (result != null) {
      final input = result['input'] as Object?;
      // _log("~~~~~~~~~~~~~monitorEvent: ${jsonEncode(monitorEvent)}");
      final isSuccessful = result['isSuccessful'] as bool?;
      final code = result['code'] as int?;
      final response = result['response'] as Object?;
      // _log("~~~~~~~~~~~~~isSuccessful: $isSuccessful");
      // _log("~~~~~~~~~~~~~code: $code");
      // _log("~~~~~~~~~~~~~response: ${jsonEncode(response)}");

      // if (kDebugMode) {
      //   await Future.delayed(const Duration(seconds: 2));
      // }

      if (response != null) {
        onResponse?.call({
          if (input != null) "input": input,
          if (isSuccessful != null) "isSuccessful": isSuccessful,
          if (code != null) "code": code,
          "response": response,
        });
      }
      final error = result['error'] as String?;
      if (error != null) {
        onFailure?.call({
          if (input != null) "input": input,
          "error": error,
        });
      }
    }
  }
}
